/*******************************************************************************
  USB Host GEN Client Driver ACM specific functions Implementation

  Company:
    Microchip Technology Inc.

  File Name:
    usb_host_gen.c

  Summary:
    USB Host GEN Client Driver ACM Specific Functions Implementation

  Description:
    This file contains the implementation of the GEN Client Driver ACM specific
    API. It should be included in the application if the GEN Host Client Driver
    ACM functionality is desired.
*******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
* Copyright (C) 2018 Microchip Technology Inc. and its subsidiaries.
*
* Subject to your compliance with these terms, you may use Microchip software
* and any derivatives exclusively with Microchip products. It is your
* responsibility to comply with third party license terms applicable to your
* use of third party software (including open source software) that may
* accompany Microchip software.
*
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
* EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
* WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
* PARTICULAR PURPOSE.
*
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
* FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
* ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
* THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *******************************************************************************/

#include "usb/usb_host_gen.h"
#include "usb/usb_host_client_driver.h"
#include "usb/usb_host.h"
#include "usb/usb_gen.h"
#include "usb/src/usb_host_gen_local.h"

// ****************************************************************************
// ****************************************************************************
// Local Functions
// ****************************************************************************
// ****************************************************************************

// *****************************************************************************
/* Function:
   void _USB_HOST_GEN_ControlTransferCallback
    (
        USB_HOST_DEVICE_OBJ_HANDLE deviceObjHandle,
        USB_HOST_REQUEST_HANDLE requestHandle,
        USB_HOST_RESULT result,
        size_t size,
        uintptr_t context
    );

  Summary:
    This function is called when a control transfer completes.

  Description:
    This function is called when a control transfer completes.

  Remarks:
    This is a local function and should not be called directly by the
    application.
*/

void _USB_HOST_GEN_ControlTransferCallback
(
    USB_HOST_DEVICE_OBJ_HANDLE deviceObjHandle,
    USB_HOST_REQUEST_HANDLE requestHandle,
    USB_HOST_RESULT result,
    size_t size,
    uintptr_t context
)
{
    USB_HOST_GEN_INSTANCE_OBJ * genInstance = (USB_HOST_GEN_INSTANCE_OBJ *)(context);
    USB_HOST_GEN_CONTROL_TRANSFER_OBJ * controlTransferObj = &genInstance->controlTransferObj;
    USB_HOST_GEN_CONTROL_REQUEST_EVENT_DATA controlRequestEventData;;

    /* The control request is complete. The request in requestType is the same
     * as the event that needs to be sent to the application. */

    controlRequestEventData.result = _USB_HOST_GEN_HostResutlToGENResultMap(result);
    controlRequestEventData.requestHandle = requestHandle;
    controlRequestEventData.length = size;
    if(genInstance->eventHandler != NULL)
    {
        genInstance->eventHandler((USB_HOST_GEN_HANDLE)(genInstance), controlTransferObj->requestType,
                &controlRequestEventData, genInstance->context);
    }

    /* Release the control transfer object */
    controlTransferObj->inUse = false;
}


// ****************************************************************************
// ****************************************************************************
// Public Functions
// ****************************************************************************
// ****************************************************************************

// ****************************************************************************
/* Function:
    USB_HOST_GEN_RESULT USB_HOST_GEN_ACM_LineCodingSet
    (
        USB_HOST_GEN_HANDLE handle, 
        USB_HOST_GEN_REQUEST_HANDLE * requestHandle, 
        USB_GEN_LINE_CODING * lineCoding
    );
       
  Summary:
    This function sends a request to the attached device to set its Line Coding.

  Description:
    This function sends a request to the attached device to set its line coding.
    The function schedules a SET LINE CODING control transfer. If successful,
    the requestHandle parameter will contain a valid request handle, else it
    will contain USB_HOST_GEN_REQUEST_HANDLE_INVALID. When completed, the GEN
    client driver will generate a USB_HOST_GEN_EVENT_ACM_SET_LINE_CODING_COMPLETE
    event. 

  Remarks:
    Refer to usb_host_gen_acm.h for usage information.
*/

USB_HOST_GEN_RESULT USB_HOST_GEN_ACM_LineCodingSet
(
    USB_HOST_GEN_HANDLE handle, 
    USB_HOST_GEN_REQUEST_HANDLE * requestHandle, 
        uint8_t * lineCoding,
        uint32_t length
)
{
    USB_HOST_GEN_RESULT result = USB_HOST_GEN_RESULT_FAILURE;
    USB_HOST_RESULT hostResult;
    USB_HOST_GEN_INSTANCE_OBJ * genInstance;
    USB_HOST_GEN_REQUEST_HANDLE * tempRequestHandle, internalRequestHandle;
    USB_SETUP_PACKET * setupPacket;

    if(requestHandle == NULL)
    {
        /* If the provided request handle is NULL, then use a temporary request
         * handle */
        tempRequestHandle = &internalRequestHandle;
    }
    else
    {
        /* Or use the actual request handle */
        tempRequestHandle = requestHandle;
    }

    genInstance = (USB_HOST_GEN_INSTANCE_OBJ *)(handle);

    if(genInstance == NULL)
    {
        /* The handle is not valid */
        result = USB_HOST_GEN_RESULT_HANDLE_INVALID;
    }
    else 
    {
        if(lineCoding == NULL)
        {
            /* The input parameter is not valid */
            result = USB_HOST_GEN_RESULT_INVALID_PARAMETER;
        }
        else
        {
            if(!genInstance->inUse)
            {
                /* This device is not valid */
                result = USB_HOST_GEN_RESULT_DEVICE_UNKNOWN;
            }
            else
            {
                if((genInstance->controlTransferObj.inUse == true))
                {
                    /* The instance is busy */
                    result = USB_HOST_GEN_RESULT_BUSY;
                }
                else
                {
                    if(genInstance->state == USB_HOST_GEN_STATE_READY)
                    {
                        genInstance->controlTransferObj.inUse = true;
                        genInstance->controlTransferObj.requestType = USB_HOST_GEN_EVENT_ACM_SET_LINE_CODING_COMPLETE;

                        /* Create the setup packet */
                        setupPacket = &genInstance->setupPacket;
                        setupPacket->bmRequestType = USB_SETUP_DIRN_HOST_TO_DEVICE | USB_SETUP_TYPE_CLASS | USB_SETUP_RECIPIENT_INTERFACE;
                        //0x21;//interface , Host to device , class
                        setupPacket->bRequest = 0; //USB_GEN_REQUEST_SET_LINE_CODING;
                        setupPacket->wValue = 0;
                        setupPacket->wIndex = genInstance->commInterfaceNumber;
                        setupPacket->wLength = length; //sizeof(USB_GEN_LINE_CODING);

                        /* Schedule the control transfer */
                        hostResult = USB_HOST_DeviceControlTransfer(genInstance->controlPipeHandle, tempRequestHandle,
                                setupPacket, lineCoding, _USB_HOST_GEN_ControlTransferCallback, 
                                (uintptr_t)(genInstance ));

                        /* Map the host result to GEN result */
                        result = _USB_HOST_GEN_HostResutlToGENResultMap(hostResult);
                        if(hostResult != USB_HOST_RESULT_SUCCESS)
                        {
                            /* This means the transfer did not go through. We 
                             * should return the control transfer object so
                             * that control transfers can be re-attempted. */
                            
                            genInstance->controlTransferObj.inUse = false;
                        }
                    }
                }
            }
        }
    }

    return(result);
}

// ****************************************************************************
/* Function:
    USB_HOST_GEN_RESULT USB_HOST_GEN_ACM_LineCodingGet
    (
        USB_HOST_GEN_HANDLE handle, 
        USB_HOST_GEN_REQUEST_HANDLE * requestHandle, 
        USB_GEN_LINE_CODING * lineCoding
    );
   
  Summary:
    This function sends a request to the attached device to get its Line Coding.

  Description:
    This function sends a request to the attached device to get its line coding.
    The function schedules a GET LINE CODING control transfer. If successful,
    the requestHandle parameter will contain a valid request handle, else it
    will contain USB_HOST_GEN_REQUEST_HANDLE_INVALID. When completed, the GEN
    client driver will generate a USB_HOST_GEN_EVENT_ACM_GET_LINE_CODING_COMPLETE
    event. 

  Remarks:
    Refer to usb_host_gen_acm.h for usage information.
*/

USB_HOST_GEN_RESULT USB_HOST_GEN_ACM_LineCodingGet
(
    USB_HOST_GEN_HANDLE handle, 
    USB_HOST_GEN_REQUEST_HANDLE * requestHandle, 
    USB_GEN_LINE_CODING * lineCoding
)
{
    USB_HOST_GEN_RESULT result = USB_HOST_GEN_RESULT_FAILURE;
    USB_HOST_RESULT hostResult;
    USB_HOST_GEN_INSTANCE_OBJ * genInstance;
    USB_HOST_GEN_REQUEST_HANDLE * tempRequestHandle, internalRequestHandle;
    USB_SETUP_PACKET * setupPacket;

    if(requestHandle == NULL)
    {
        /* If the provided request handle is NULL, then use a temporary request
         * handle */
        tempRequestHandle = &internalRequestHandle;
    }
    else
    {
        /* Or use the actual request handle */
        tempRequestHandle = requestHandle;
    }

    genInstance = (USB_HOST_GEN_INSTANCE_OBJ *)(handle);

    if(genInstance == NULL)
    {
        /* The handle is not valid */
        result = USB_HOST_GEN_RESULT_HANDLE_INVALID;
    }
    else 
    {
        if(lineCoding == NULL)
        {
            /* The input parameter is not valid */
            result = USB_HOST_GEN_RESULT_INVALID_PARAMETER;
        }
        else
        {
            if(!genInstance->inUse)
            {
                /* This device is not valid */
                result = USB_HOST_GEN_RESULT_DEVICE_UNKNOWN;
            }
            else
            {
                if((genInstance->controlTransferObj.inUse == true))
                {
                    /* The instance is busy */
                    result = USB_HOST_GEN_RESULT_BUSY;
                }
                else
                {
                    if(genInstance->state == USB_HOST_GEN_STATE_READY)
                    {
                        genInstance->controlTransferObj.inUse = true;
                        genInstance->controlTransferObj.requestType = USB_HOST_GEN_EVENT_ACM_GET_LINE_CODING_COMPLETE;

                        /* Create the setup packet */
                        setupPacket = &genInstance->setupPacket;
                        setupPacket->bmRequestType = 0xA1;
                        setupPacket->bRequest = USB_GEN_REQUEST_GET_LINE_CODING;
                        setupPacket->wValue = 0;
                        setupPacket->wIndex = genInstance->commInterfaceNumber;
                        setupPacket->wLength = sizeof(USB_GEN_LINE_CODING);

                        /* Schedule the control transfer */
                        hostResult = USB_HOST_DeviceControlTransfer(genInstance->controlPipeHandle, tempRequestHandle,
                                setupPacket, lineCoding, _USB_HOST_GEN_ControlTransferCallback, 
                                (uintptr_t)(genInstance ));

                        /* Map the host result to GEN result */
                        result = _USB_HOST_GEN_HostResutlToGENResultMap(hostResult);
                        if(hostResult != USB_HOST_RESULT_SUCCESS)
                        {
                            /* This means the transfer did not go through. We 
                             * should return the control transfer object so
                             * that control transfers can be re-attempted. */
                            
                            genInstance->controlTransferObj.inUse = false;
                        }
                    }
                }
            }
        }
    }

    return(result);
}

// ****************************************************************************
/* Function:
    USB_HOST_GEN_RESULT USB_HOST_GEN_ACM_BreakSend
    (
        USB_HOST_GEN_HANDLE handle,
        USB_HOST_GEN_REQUEST_HANDLE * requestHandle,
        uint16_t breakDuration
    );
   
  Summary:
    This function sends a request to the attached device to update its break
    duration.

  Description:
    This function sends a request to the attached to update its break
    duration. The function schedules a SEND BREAK control transfer. If
    successful, the transferHandle parameter will contain a valid request
    handle, else it will contain USB_HOST_GEN_REQUEST_HANDLE_INVALID. When
    completed, the GEN client driver will generate a
    USB_HOST_GEN_EVENT_ACM_SEND_BREAK_COMPLETE event. 

  Remarks:
    Refer to usb_host_gen_acm.h for usage information.
*/

USB_HOST_GEN_RESULT USB_HOST_GEN_ACM_BreakSend
(
    USB_HOST_GEN_HANDLE handle,
    USB_HOST_GEN_REQUEST_HANDLE * requestHandle,
    uint16_t breakDuration
)
{
    USB_HOST_GEN_RESULT result = USB_HOST_GEN_RESULT_FAILURE;
    USB_HOST_RESULT hostResult;
    USB_HOST_GEN_INSTANCE_OBJ * genInstance;
    USB_HOST_GEN_REQUEST_HANDLE * tempRequestHandle, internalRequestHandle;
    USB_SETUP_PACKET * setupPacket;

    if(requestHandle == NULL)
    {
        /* If the provided request handle is NULL, then use a temporary request
         * handle */
        tempRequestHandle = &internalRequestHandle;
    }
    else
    {
        /* Or use the actual request handle */
        tempRequestHandle = requestHandle;
    }

    genInstance = (USB_HOST_GEN_INSTANCE_OBJ *)(handle);

    if(genInstance == NULL)
    {
        /* The handle is not valid */
        result = USB_HOST_GEN_RESULT_HANDLE_INVALID;
    }
    else 
    {
        if(!genInstance->inUse)
        {
            /* This device is not valid */
            result = USB_HOST_GEN_RESULT_DEVICE_UNKNOWN;
        }
        else
        {
            if((genInstance->controlTransferObj.inUse == true))
            {
                /* The instance is busy */
                result = USB_HOST_GEN_RESULT_BUSY;
            }
            else
            {
                if(genInstance->state == USB_HOST_GEN_STATE_READY)
                {
                    genInstance->controlTransferObj.inUse = true;
                    genInstance->controlTransferObj.requestType = USB_HOST_GEN_EVENT_ACM_SEND_BREAK_COMPLETE;

                    /* Create the setup packet */
                    setupPacket = &genInstance->setupPacket;
                    setupPacket->bmRequestType = 0x21;
                    setupPacket->bRequest = USB_GEN_REQUEST_SEND_BREAK;
                    setupPacket->wValue = breakDuration ;
                    setupPacket->wIndex = genInstance->commInterfaceNumber;
                    setupPacket->wLength = 0;

                    /* Schedule the control transfer */
                    hostResult = USB_HOST_DeviceControlTransfer(genInstance->controlPipeHandle, tempRequestHandle,
                            setupPacket, NULL, _USB_HOST_GEN_ControlTransferCallback, 
                            (uintptr_t)(genInstance ));

                    /* Map the host result to GEN result */
                    result = _USB_HOST_GEN_HostResutlToGENResultMap(hostResult);
                    if(hostResult != USB_HOST_RESULT_SUCCESS)
                    {
                        /* If the control transfer was not successful, then
                         * release the control transfer object. */
                        genInstance->controlTransferObj.inUse = false;
                    }
                }
            }
        }

    }

    return(result);
}

// ****************************************************************************
/* Function:
    USB_HOST_GEN_RESULT USB_HOST_GEN_ACM_ControlLineStateSet
    (
        USB_HOST_GEN_HANDLE handle,
        USB_HOST_GEN_REQUEST_HANDLE * requestHandle, 
        USB_GEN_CONTROL_LINE_STATE * controlLineState
    );
   
  Summary:
    This function sends a request to the attached device to set its Control Line
    State.

  Description:
    This function sends a request to the attached to set its Control Line State.
    The function schedules a SET CONTROL LINE STATE control transfer. If
    successful, the requestHandle parameter will contain a valid request handle,
    else it will contain USB_HOST_GEN_REQUEST_HANDLE_INVALID. When completed,
    the GEN client driver will generate a
    USB_HOST_GEN_EVENT_ACM_SET_CONTROL_LINE_STATE_COMPLETE event. 

  Remarks:
    Refer to usb_host_gen_acm.h for usage information.
*/

USB_HOST_GEN_RESULT USB_HOST_GEN_ACM_ControlLineStateSet
(
    USB_HOST_GEN_HANDLE handle,
    USB_HOST_GEN_REQUEST_HANDLE * requestHandle, 
    USB_GEN_CONTROL_LINE_STATE * controlLineState
)
{
    USB_HOST_GEN_RESULT result = USB_HOST_GEN_RESULT_FAILURE;
    USB_HOST_RESULT hostResult;
    USB_HOST_GEN_INSTANCE_OBJ * genInstance;
    USB_HOST_GEN_REQUEST_HANDLE * tempRequestHandle, internalRequestHandle;
    USB_SETUP_PACKET * setupPacket;

    if(requestHandle == NULL)
    {
        /* If the provided request handle is NULL, then use a temporary request
         * handle */
        tempRequestHandle = &internalRequestHandle;
    }
    else
    {
        /* Or use the actual request handle */
        tempRequestHandle = requestHandle;
    }

    genInstance = (USB_HOST_GEN_INSTANCE_OBJ *)(handle);

    if(genInstance == NULL)
    {
        /* The handle is not valid */
        result = USB_HOST_GEN_RESULT_HANDLE_INVALID;
    }
    else 
    {
        if(controlLineState == NULL)
        {
            result = USB_HOST_GEN_RESULT_INVALID_PARAMETER;
        }
        else
        {
            if(!genInstance->inUse)
            {
                /* This device is not valid */
                result = USB_HOST_GEN_RESULT_DEVICE_UNKNOWN;
            }
            else
            {
                if((genInstance->controlTransferObj.inUse == true))
                {
                    /* The instance is busy */
                    result = USB_HOST_GEN_RESULT_BUSY;
                }
                else
                {
                    if(genInstance->state == USB_HOST_GEN_STATE_READY)
                    {
                        genInstance->controlTransferObj.inUse = true;
                        genInstance->controlTransferObj.requestType = USB_HOST_GEN_EVENT_ACM_SET_CONTROL_LINE_STATE_COMPLETE;

                        /* Create the setup packet */
                        setupPacket = &genInstance->setupPacket;
                        setupPacket->bmRequestType = 0x21;
                        setupPacket->bRequest = USB_GEN_REQUEST_SET_CONTROL_LINE_STATE;
                        setupPacket->wValue =  (( *(uint8_t *) controlLineState ) << 8 ); ;
                        setupPacket->wIndex = genInstance->commInterfaceNumber;
                        setupPacket->wLength = 0;

                        /* Schedule the control transfer */
                        hostResult = USB_HOST_DeviceControlTransfer(genInstance->controlPipeHandle, tempRequestHandle,
                                setupPacket, NULL, _USB_HOST_GEN_ControlTransferCallback, 
                                (uintptr_t)(genInstance ));

                        /* Map the host result to GEN result */
                        result = _USB_HOST_GEN_HostResutlToGENResultMap(hostResult);
                        if(hostResult != USB_HOST_RESULT_SUCCESS)
                        {
                            /* This means the transfer did not go through. We 
                             * should return the control transfer object so
                             * that control transfers can be re-attempted. */
                            
                            genInstance->controlTransferObj.inUse = false;
                        }
                    }
                }
            }
        }
    }

    return(result);
}


